
/*---------------------------------------------------------------------------

    bit.c -- bitshift functions for Fortran.

    Written By: S.E. Norris

    $RCSfile: bit.c,v $
    $Author: norris $
    $Revision: 1.5 $
    $Date: 1996/11/28 12:00:26 $
 
    $Log: bit.c,v $
    Revision 1.5  1996/11/28 12:00:26  norris
    General clean up of the code.

    Revision 1.4  1996/04/06 08:30:02  norris
    Debugged version of code.

    Revision 1.3  1996/04/06  07:46:32  norris
    Added the ibits() and ishftc() functions.

    Revision 1.2  1996/04/06  03:57:06  norris
    Added the bit_size(), btest(), ibclr() and ibset() functions.

    Revision 1.1  1996/04/06  02:22:11  norris
    Initial revision

 ---------------------------------------------------------------------------*/

#include "libU77.h"


subroutine bprint_(char *c, ftnlen l, integer *v)
{
  int i, j, mask = 1 << 31;

  j = (int) *v;
  for (i = 1; i <= 32; ++i) {
    *c = (((j &mask) == 0) ? '0' : '1');
    *c++;
    j <<= 1;
    if (i % 8 == 0 && i != 32) {
      *c = ' ';
      *c++;
    }
  }
}


integer bit_size__(integer *a)
{
  return (integer)(sizeof(*a) * 8);
}


logical btest_(integer *a, integer *p)
{
  int i = 1;

  if (((i << abs(*p)) & *a) != 0) {
    return TRUE_;
  } else {
    return FALSE_;
  }
}


integer iand_(integer *a, integer *b)
{
  return (*a & *b);
}


integer ibits_(integer *a, integer *p, integer *l)
{
  unsigned int mask = ~0;

  return ((mask >> abs(*p)) & *a) >> ((sizeof(*a)*8) - (abs(*p) + abs(*l)));
}


integer ibset_(integer *a, integer *p)
{
  int i = 1;

  return  ((i << abs(*p)) | *a);
}


integer ibclr_(integer *a, integer *p)
{
  int i = 1;

  return  (~(i << abs(*p)) & *a);
}


integer ieor_(integer *a, integer *b)
{
  return (*a ^ *b);
}


integer ior_(integer *a, integer *b)
{
  return (*a | *b);
}


integer ishft_(integer *a, integer *s)
{
  return ((*s < 0) ? *a << abs(*s) : *a >> abs(*s));
}


integer ishftc_(integer *a, integer *s)
{
  unsigned int i, k;

  i = *a;

  if (*s > 0) {
    for (k = 0;k < abs(*s);++k) {
      i = (i << 1) ^ (i >> (sizeof(i)*8 - 1));
    }
  } else {
    for (k = 0;k < abs(*s);++k) {
      i = (i >> 1) ^ (i << (sizeof(i)*8 - 1));
    }
  }
  return (integer) i;
}


integer not_(integer *a)
{
  return (~*a);
}
